﻿using System.Diagnostics;
using ASRP31NIOSSDK;
using Foundation;
using P31NMAUIDemo.Platforms.iOS;

namespace P31NMAUIDemo;

public partial class MainPage : ContentPage
{
    enum Actions
    {
        None,
        Search,
        Connect,
        Start,
        Stop
    }
    int count = 0;
    MainPageViewModel vm = new MainPageViewModel();
    System.Timers.Timer actionTimer = new System.Timers.Timer();
    Actions currentAction = Actions.None;

    public MainPage()
    {
        InitializeComponent();
        this.BindingContext = vm;
        actionTimer.Interval = 3000;
        actionTimer.AutoReset = false;
        actionTimer.Elapsed += ActionTimer_Elapsed;
    }

    private void ActionTimer_Elapsed(object? sender, System.Timers.ElapsedEventArgs e)
    {
        Utility.LoadActivityIndicator(vm, false);
        MainThread.BeginInvokeOnMainThread(() =>
        {
            if (currentAction == Actions.Connect)
            {
                DisplayAlert("Connect", "Connect Failed", "OK");
            }
        });
    }

    protected override void OnAppearing()
    {
        base.OnAppearing();
        ASRP31NManager.Shared().searchingIP = ASRP31NManagerSearchingIP;
        ASRP31NManager.Shared().changeNetworkState = ASRP31NManagerChangeNetworkState;

        ASRP31NManager.Shared().receivedPCEPCData = ASRP31NManagerReceivedPCEPCData;
        //ASRP31NManager.Shared().StartSearchDevice();
        List<bool> ants = Utility.GetInventoryAnts();
        InitInventoryAnts(ants);
    }

    void InitInventoryAnts(List<bool> ants)
    {
        if (ants == null || ants.Count == 0) return;
        ckAnt1.IsChecked = ants[0];
        ckAnt2.IsChecked = ants[1];
        ckAnt3.IsChecked = ants[2];
        ckAnt4.IsChecked = ants[3];
        ckAnt5.IsChecked = ants[4];
        ckAnt6.IsChecked = ants[5];
        ckAnt7.IsChecked = ants[6];
        ckAnt8.IsChecked = ants[7];
    }

    private void OnCounterClicked(object sender, EventArgs e)
    {
        if (count == 2)
        {
            ASRP31NManager.Shared().StartMultiReadTags(0, 0, 0, true, true, true, true, true, true, true, true, false, ASRP31NSDKOperationMode.Multi);
        }
        else if (count == 3)
        {
            ASRP31NManager.Shared().StopReadTags();
        }
        else if (count == 4)
        {
            ASRP31NManager.Shared().DisconnectServer();
        }

    }
    private String currentip = "";
    public void ASRP31NManagerSearchingIP(string ip, bool isFinish)
    {
        Console.WriteLine("MainPage ASRP31NManagerSearchingIP ip:" + ip + " isFinish:" + isFinish);
        if (ip != "" && ip != null)
        {
            vm.IPList.Add(ip);
        }
        else if (isFinish)
        {
            Utility.LoadActivityIndicator(vm, false);
            if (vm.IPList.Count == 0)
            {
                MainThread.BeginInvokeOnMainThread(() =>
                {
                    DisplayAlert("Search", "Not found devices", "OK");
                });
            }
            else
            {
                ShowIPPicker(true);
            }
        }
    }

    public void ASRP31NManagerChangeNetworkState(ASRP31NSDKNetworkState state, NSError error)
    {
        Debug.WriteLine("MainPage ASRP31NManagerChangeNetworkState state:" + state);
        //if (currentAction == Actions.Search) return;//search前先断开
        string msg = "";
        if (state == ASRP31NSDKNetworkState.Connected)
        {
            vm.IsConnected = true;
            vm.SelectedIP = selectedIP;
            msg = "Connected";
            //Utility.ShowToastMessage(vm, "Connected");
            //MainThread.BeginInvokeOnMainThread(() =>
            //{
            //    btnRead_Clicked(btnRead, new EventArgs());
            //});
        }
        else if (state == ASRP31NSDKNetworkState.Error)
        {
            vm.IsConnected = false;
            vm.SelectedIP = "Disconnected";
            msg = "Connect failed";
        }
        else if (state == ASRP31NSDKNetworkState.Disconnected)
        {
            Disconnected();
            msg = "Disconnected";
        }

        if (currentAction == Actions.Connect)
        {
            actionTimer.Stop();
            Utility.LoadActivityIndicator(vm, false);
            Utility.ShowToastMessage(vm, msg);
        }
    }

    void Disconnected()
    {
        vm.IsConnected = false;
        vm.SelectedIP = "Disconnected";
        vm.Tags.Clear();
        vm.TotalTags = 0;
    }

    public void ASRP31NManagerReceivedPCEPCData(byte[] pcEpc, int antenna)
    {
        MainThread.BeginInvokeOnMainThread(() =>
        {
            try
            {
                if (btnRead.IsEnabled && currentAction != Actions.Stop) EnableReadButton(false);
                string currentEpc = BitConverter.ToString(pcEpc).Replace("-", "");
                Debug.WriteLine("MainPage ASRP31NManagerReceivedPCEPCData pcEpc:" + currentEpc + "antenna:" + antenna);
                //vm.ShowLabelTags = true;
                var tag = vm.Tags.FirstOrDefault(t => t.EPC == currentEpc);
                if (tag == null)
                {
                    vm.Tags.Add(new Tag(currentEpc, antenna));
                    vm.TotalTags++;
                }
                else
                {
                    if (tag.Ant != antenna) tag.Ant = antenna;
                    tag.Count++;
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine("****************" + ex.Message);
            }
        });
    }

    void btnSearch_Clicked(System.Object sender, System.EventArgs e)
    {
        if (!btnRead.IsEnabled)
            btnStop_Clicked(btnStop, new EventArgs());
        if (vm.IsConnected)
            ASRP31NManager.Shared().DisconnectServer();
        currentAction = Actions.Search;
        vm.IPList.Clear();
        Utility.LoadActivityIndicator(vm, true, "Search...");
        ASRP31NManager.Shared().StartSearchDevice();
    }

    void EnableReadButton(bool isEnabled)
    {
        btnRead.IsEnabled = isEnabled;
        btnRead.TextColor = isEnabled ? Color.FromArgb("#007aff") : Colors.LightGray;
        btnRead.BackgroundColor = Colors.Transparent;
        vm.AntIsEnabled = isEnabled;
    }

    void btnRead_Clicked(System.Object sender, System.EventArgs e)
    {
        if (ckAnt8.IsChecked || ckAnt7.IsChecked || ckAnt6.IsChecked || ckAnt5.IsChecked || ckAnt4.IsChecked || ckAnt3.IsChecked || ckAnt2.IsChecked || ckAnt1.IsChecked)
        {
            currentAction = Actions.Start;
            EnableReadButton(false);
            ASRP31NManager.Shared().StartMultiReadTags(0, 0, 0, ckAnt1.IsChecked, ckAnt2.IsChecked, ckAnt3.IsChecked, ckAnt4.IsChecked, ckAnt5.IsChecked, ckAnt6.IsChecked, ckAnt7.IsChecked, ckAnt8.IsChecked, false, ASRP31NSDKOperationMode.Multi);
            Utility.SetInventoryAnts(new List<bool> { ckAnt1.IsChecked, ckAnt2.IsChecked, ckAnt3.IsChecked, ckAnt4.IsChecked, ckAnt5.IsChecked, ckAnt6.IsChecked, ckAnt7.IsChecked, ckAnt8.IsChecked });
        }
        else
        {
            DisplayAlert("", "Please select Antenna", "OK");
        }
    }

    void btnClear_Clicked(System.Object sender, System.EventArgs e)
    {
        vm.Tags.Clear();
        vm.ShowLabelTags = false;
        vm.TotalTags = 0;
    }

    void btnStop_Clicked(System.Object sender, System.EventArgs e)
    {
        currentAction = Actions.Stop;
        ASRP31NManager.Shared().StopReadTags();
        Thread.Sleep(300);
        EnableReadButton(true);//在返回标签的代理中会将read按钮置灰，操作有影响出bug所以要sleep
    }

    void btnCancel_Clicked(System.Object sender, System.EventArgs e)
    {
        ShowIPPicker(false);
    }

    void ShowIPPicker(bool show)
    {
        vm.ShowIPList = show;
        vm.ShowMask = show;
    }

    string selectedIP = "";
    void btnConnect_Clicked(System.Object sender, System.EventArgs e)
    {
        ShowIPPicker(false);
        selectedIP = (string)(((Button)sender).CommandParameter);
        Utility.LoadActivityIndicator(vm, true, "Connecting...");
        currentAction = Actions.Connect;
        actionTimer.Start();
        ASRP31NManager.Shared().ConnectServer(selectedIP, 5000);
    }

    async void ToolbarMore_Clicked(System.Object sender, System.EventArgs e)
    {
        await Navigation.PushAsync(new InfoPage());
    }
}


