﻿using System;
using ASRP31NIOSSDK;
using Foundation;
using static System.Runtime.InteropServices.JavaScript.JSType;

namespace P31NMAUIDemo.Platforms.iOS
{
    public class ASRP31NManager
    {
        public delegate void ASRP31NManagerChangeNetworkState(ASRP31NSDKNetworkState state, NSError error);
        public ASRP31NManagerChangeNetworkState? changeNetworkState;

        public delegate void ASRP31NManagerReaderConnectedStatus(ASRP31NSDKCommonStatus statusCode);
        public ASRP31NManagerReaderConnectedStatus? readerConnectedStatus;

        public delegate void ASRP31NManagerSearchingIP(string ip, bool isFinish);
        public ASRP31NManagerSearchingIP? searchingIP;

        public delegate void ASRP31NManagerReceivedPCEPCData(byte[] pcEpc, int antenna);
        public ASRP31NManagerReceivedPCEPCData? receivedPCEPCData;


        public class ASRP31NManagerDelegate : ASRP31NSDKDelegate
        {
            public ASRP31NManagerDelegate()
            {

            }
            public override void ChangeNetworkState(ASRP31NSDK ASRP31N, ASRP31NSDKNetworkState state, NSError error)
            {
                ASRP31NManager.Shared().changeNetworkState?.Invoke(state, error);
            }
            public override void SearchingIP(ASRP31NSDK ASRP31N, string ip, bool isFinish)
            {
                Console.WriteLine("SearchingIP ip:" + ip);
                ASRP31NManager.Shared().searchingIP?.Invoke(ip, isFinish);
            }
            public override void ReceivedPCEPCData(ASRP31NSDK ASRP31N, NSData pcEpc, int antenna)
            {
                byte[] bytes = new byte[pcEpc.Length];
                bytes = pcEpc.ToArray();
                ASRP31NManager.Shared().receivedPCEPCData?.Invoke(bytes, antenna);
            }
        }

        static ASRP31NManager? instance = null;
        public ASRP31NSDK aSRP31NSDK;
        public ASRP31NManagerDelegate aSRP31NManagerDelegate;
        public static ASRP31NManager Shared()
        {
            if (instance == null)
            {
                instance = new ASRP31NManager();
            }
            return instance;
        }
        public ASRP31NManager()
        {
            aSRP31NSDK = ASRP31NSDK.SharedInstance;
            aSRP31NManagerDelegate = new ASRP31NManagerDelegate();
            aSRP31NSDK.Delegate = aSRP31NManagerDelegate;
        }

        public bool StartSearchDevice()
        {
            NSError error = aSRP31NSDK.StartSearchDevice;
            if (error == null)
            {

                return true;
            } else
            {
                Console.WriteLine("StartSearchDevice error:" + error?.Description);
                return false;
            }
        }

        public bool StopSearchDevice()
        {
            NSError error = aSRP31NSDK.StopSearchDevice;
            if (error == null)
            {
                return true;
            }
            else
            {
                Console.WriteLine("StopSearchDevice error:" + error?.Description);
                return false;
            }
        }
        public bool StartMultiReadTags(
            byte mtnu,
            byte mtime,
            ushort repeatCycle,
            bool enable1,
            bool enable2,
            bool enable3,
            bool enable4,
            bool enable5,
            bool enable6,
            bool enable7,
            bool enable8,
            bool isOnRSSI,
            ASRP31NSDKOperationMode operationMode)
        {
            if (aSRP31NSDK.ConnectionStatus == false)
            {
                return false;
            }
            NSError error = aSRP31NSDK.StartMultiReadTags(
                mtnu,
                mtime,
                repeatCycle,
                enable1,
                enable2,
                enable3,
                enable4,
                enable5,
                enable6,
                enable7,
                enable8,
                isOnRSSI,
                operationMode);
            if (error == null)
            {
                return true;
            }
            else
            {
                Console.WriteLine("StartMultiReadTags error:" + error?.Description);
                return false;
            }
        }
        public bool StopReadTags()
        {
            if (aSRP31NSDK.ConnectionStatus == false)
            {
                return false;
            }
            NSError error = aSRP31NSDK.StopReadTags;
            if (error == null)
            {
                return true;
            }
            else
            {
                Console.WriteLine("StopReadTags error:" + error?.Description);
                return false;
            }
        }
        public bool ConnectServer(
            string ip,
            int port)
        {

            NSError error = aSRP31NSDK.ConnectServer(ip, port);
            if (error == null)
            {
                return true;
            }
            else
            {
                Console.WriteLine("ConnectServer error:" + error?.Description);
                return false;
            }
        }
        public bool DisconnectServer()
        {
            if (aSRP31NSDK.ConnectionStatus == false)
            {
                return false;
            }
            NSError error = aSRP31NSDK.DisconnectServer;
            if (error == null)
            {
                return true;
            }
            else
            {
                Console.WriteLine("DisconnectServer error:" + error?.Description);
                return false;
            }
        }
        public string GetSDKVersion()
        {
            return aSRP31NSDK.SDKVersion;
        }
        public string GetDLLVersion()
        {
            return ASRP31NIOSSDKDLL.DLLVersion();
        }
    }
}

